@@ -1,5 +1,6 @@
 import functools
 import logging
+from datetime import datetime
 from django.http import HttpResponse
 from django.conf.urls import patterns, url
 from django.core.urlresolvers import reverse
@@ -8,14 +9,14 @@ from restlib2.http import codes
 from preserialize.serialize import serialize
 from avocado.events import usage
 from avocado.models import DataContext
-from avocado.conf import settings
 from serrano.forms import ContextForm
-from .base import BaseResource
+from .base import ThrottledResource
+from .history import RevisionsResource, ObjectRevisionsResource, \
+    ObjectRevisionResource
 from . import templates
 
 log = logging.getLogger(__name__)
 
-HISTORY_ENABLED = settings.HISTORY_ENABLED
 
 def context_posthook(instance, data, request):
     uri = request.build_absolute_uri
@@ -29,13 +30,14 @@ def context_posthook(instance, data, request):
 
     data['_links'] = {
         'self': {
-            'href': uri(reverse('serrano:contexts:single', args=[instance.pk])),
+            'href': uri(
+                reverse('serrano:contexts:single', args=[instance.pk])),
         }
     }
     return data
 
 
-class ContextBase(BaseResource):
+class ContextBase(ThrottledResource):
     cache_max_age = 0
     private_cache = True
 
@@ -80,9 +82,9 @@ class ContextBase(BaseResource):
 
 
 class ContextsResource(ContextBase):
-    "Resource of active (non-archived) contexts"
+    "Resource of contexts"
     def get(self, request):
-        queryset = self.get_queryset(request, archived=False)
+        queryset = self.get_queryset(request)
 
         # Only create a default if a session exists
         if request.session.session_key:
@@ -99,23 +101,16 @@ class ContextsResource(ContextBase):
         form = ContextForm(request, request.data)
 
         if form.is_valid():
-            instance = form.save(archive=HISTORY_ENABLED)
+            instance = form.save()
             usage.log('create', instance=instance, request=request)
             response = self.render(request, self.prepare(request, instance),
-                status=codes.created)
+                                   status=codes.created)
         else:
             response = self.render(request, dict(form.errors),
-                status=codes.unprocessable_entity)
+                                   status=codes.unprocessable_entity)
         return response
 
 
-class ContextsHistoryResource(ContextBase):
-    "Resource of archived (non-active) contexts"
-    def get(self, request):
-        queryset = self.get_queryset(request, archived=True)
-        return self.prepare(request, queryset)
-
-
 class ContextResource(ContextBase):
     "Resource for accessing a single context"
     def get_object(self, request, pk=None, session=None, **kwargs):
@@ -140,6 +135,8 @@ class ContextResource(ContextBase):
 
     def get(self, request, **kwargs):
         usage.log('read', instance=request.instance, request=request)
+        self.model.objects.filter(pk=request.instance.pk).update(
+            accessed=datetime.now())
         return self.prepare(request, request.instance)
 
     def put(self, request, **kwargs):
@@ -147,32 +144,47 @@ class ContextResource(ContextBase):
         form = ContextForm(request, request.data, instance=instance)
 
         if form.is_valid():
-            instance = form.save(archive=HISTORY_ENABLED)
+            instance = form.save()
             usage.log('update', instance=instance, request=request)
             response = self.render(request, self.prepare(request, instance))
         else:
             response = self.render(request, dict(form.errors),
-                status=codes.unprocessable_entity)
+                                   status=codes.unprocessable_entity)
         return response
 
     def delete(self, request, **kwargs):
         if request.instance.session:
             return HttpResponse(status=codes.bad_request)
         request.instance.delete()
-        usage.log('delete', instance=instance, request=request)
+        usage.log('delete', instance=request.instance, request=request)
         return HttpResponse(status=codes.no_content)
 
 
 single_resource = never_cache(ContextResource())
 active_resource = never_cache(ContextsResource())
-history_resource = never_cache(ContextsHistoryResource())
+revisions_resource = never_cache(RevisionsResource(
+    object_model=DataContext, object_model_template=templates.Context,
+    object_model_base_uri='serrano:contexts'))
+revisions_for_object_resource = never_cache(ObjectRevisionsResource(
+    object_model=DataContext, object_model_template=templates.Context,
+    object_model_base_uri='serrano:contexts'))
+revision_for_object_resource = never_cache(ObjectRevisionResource(
+    object_model=DataContext, object_model_template=templates.Context,
+    object_model_base_uri='serrano:contexts'))
 
 # Resource endpoints
-urlpatterns = patterns('',
+urlpatterns = patterns(
+    '',
     url(r'^$', active_resource, name='active'),
-    url(r'^history/$', history_resource, name='history'),
 
     # Endpoints for specific contexts
     url(r'^(?P<pk>\d+)/$', single_resource, name='single'),
     url(r'^session/$', single_resource, {'session': True}, name='session'),
+
+    # Revision related endpoints
+    url(r'^revisions/$', revisions_resource, name='revisions'),
+    url(r'^(?P<pk>\d+)/revisions/$', revisions_for_object_resource,
+        name='revisions_for_object'),
+    url(r'^(?P<object_pk>\d+)/revisions/(?P<revision_pk>\d+)/$',
+        revision_for_object_resource, name='revision_for_object'),
 )
