@@ -1,10 +1,9 @@
 import json
 from decimal import Decimal
-from collections import defaultdict
 from django.db.models import Q
 from django.http import HttpResponse
 from restlib2.http import codes
-from restlib2.params import Parametizer, param_cleaners
+from restlib2.params import Parametizer, StrParam, BoolParam, IntParam
 from modeltree.tree import MODELTREE_DEFAULT_ALIAS, trees
 from avocado.models import DataField
 from avocado.stats import kmeans
@@ -17,36 +16,12 @@ MAXIMUM_OBSERVATIONS = 50000
 
 
 class FieldDistParametizer(Parametizer):
-    tree = MODELTREE_DEFAULT_ALIAS
-    aware = False
-    nulls = False
-    sort = None
-    cluster = True
-    n = None
-
-    # Not implemented
-    aggregates = None
-    relative = None
-
-    def clean_tree(self, value):
-        if value not in trees:
-            return MODELTREE_DEFAULT_ALIAS
-        return value
-
-    def clean_aware(self, value):
-        return param_cleaners.clean_bool(value)
-
-    def clean_nulls(self, value):
-        return param_cleaners.clean_bool(value)
-
-    def clean_sort(self, value):
-        return param_cleaners.clean_string(value)
-
-    def clean_cluster(self, value):
-        return param_cleaners.clean_bool(value)
-
-    def clean_n(self, value):
-        return param_cleaners.clean_int(value)
+    tree = StrParam(MODELTREE_DEFAULT_ALIAS, choices=trees)
+    aware = BoolParam(False)
+    nulls = BoolParam(False)
+    sort = StrParam()
+    cluster = BoolParam(True)
+    n = IntParam()
 
 
 class FieldDistribution(FieldBase):
@@ -60,8 +35,9 @@ class FieldDistribution(FieldBase):
 
         tree = trees[params.get('tree')]
         opts = tree.root_model._meta
-        tree_field = DataField(app_name=opts.app_label,
-            model_name=opts.module_name, field_name=opts.pk.name)
+        tree_field = DataField(
+            app_name=opts.app_label, model_name=opts.module_name,
+            field_name=opts.pk.name)
 
         # This will eventually make it's way in the parametizer, but lists
         # are not supported
@@ -131,12 +107,13 @@ class FieldDistribution(FieldBase):
 
         if length > MAXIMUM_OBSERVATIONS:
             return HttpResponse(json.dumps({'error': 'Data too large'}),
-                status=codes.unprocessable_entity)
+                                status=codes.unprocessable_entity)
 
         # Apply ordering. If any of the fields are enumerable, ordering should
         # be relative to those fields. For continuous data, the ordering is
         # relative to the count of each group
-        if any([d.enumerable for d in fields]) and not params['sort'] == 'count':
+        if (any([d.enumerable for d in fields]) and
+                not params['sort'] == 'count'):
             stats = stats.order_by(*groupby)
         else:
             stats = stats.order_by('-count')
@@ -163,7 +140,8 @@ class FieldDistribution(FieldBase):
                 clustered = True
 
                 counts = [p['count'] for p in points]
-                points, outliers = kmeans.weighted_counts(obs, counts, params['n'])
+                points, outliers = kmeans.weighted_counts(
+                    obs, counts, params['n'])
             else:
                 indexes = kmeans.find_outliers(obs, normalized=False)
 
